
/* Chrysalide - Outil d'analyse de fichiers binaires
 * enigma-murmur.c - transormation en empreinte d'API enigma-murmur
 *
 * Copyright (C) 2023 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "enigma-murmur.h"


#include <malloc.h>


#include <analysis/scan/patterns/modifier-int.h>




/* ----------------------- RECHERCHE D'UN MOTIF DE TEXTE BRUT ----------------------- */


/* Initialise la classe des empreintes enigma-murmur. */
static void g_scan_enigma_murmur_modifier_class_init(GScanEnigmaMurmurModifierClass *);

/* Initialise une instance d'empreinte enigma-murmur. */
static void g_scan_enigma_murmur_modifier_init(GScanEnigmaMurmurModifier *);

/* Supprime toutes les références externes. */
static void g_scan_enigma_murmur_modifier_dispose(GScanEnigmaMurmurModifier *);

/* Procède à la libération totale de la mémoire. */
static void g_scan_enigma_murmur_modifier_finalize(GScanEnigmaMurmurModifier *);



/* --------------------- IMPLEMENTATION DES FONCTIONS DE CLASSE --------------------- */


/* Fournit le nom d'appel d'un modificateur pour motif. */
static char *g_scan_enigma_murmur_modifier_get_name(const GScanEnigmaMurmurModifier *);

/* Calcule l'empreinte enigma-murmur d'un motif de recherche. */
static uint32_t compute_enigma_murmur_hash32(const sized_binary_t *);

/* Transforme une séquence d'octets pour motif de recherche. */
static bool g_scan_enigma_murmur_modifier_transform(const GScanEnigmaMurmurModifier *, const sized_binary_t *, size_t, sized_binary_t **, size_t *);

/* Retrouve l'origine d'une correspondance à partir d'un indice. */
static char *g_scan_enigma_murmur_modifier_get_path(const GScanEnigmaMurmurModifier *, size_t *);



/* ---------------------------------------------------------------------------------- */
/*                         RECHERCHE D'UN MOTIF DE TEXTE BRUT                         */
/* ---------------------------------------------------------------------------------- */


/* Indique le type défini pour une transormation en empreinte enigma-murmur. */
G_DEFINE_TYPE(GScanEnigmaMurmurModifier, g_scan_enigma_murmur_modifier, G_TYPE_API_HASH_MODIFIER);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des empreintes enigma-murmur.           *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_scan_enigma_murmur_modifier_class_init(GScanEnigmaMurmurModifierClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */
    GScanTokenModifierClass *modifier;      /* Version de classe parente   */

    object = G_OBJECT_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_scan_enigma_murmur_modifier_dispose;
    object->finalize = (GObjectFinalizeFunc)g_scan_enigma_murmur_modifier_finalize;

    modifier = G_SCAN_TOKEN_MODIFIER_CLASS(klass);

    modifier->get_name = (get_scan_modifier_name_fc)g_scan_enigma_murmur_modifier_get_name;

    modifier->transform = (transform_scan_token_fc)g_scan_enigma_murmur_modifier_transform;
    modifier->get_path = (get_modifier_path)g_scan_enigma_murmur_modifier_get_path;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : modifier = instance à initialiser.                           *
*                                                                             *
*  Description : Initialise une instance d'empreinte enigma-murmur.           *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_scan_enigma_murmur_modifier_init(GScanEnigmaMurmurModifier *modifier)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : modifier = instance d'objet GLib à traiter.                  *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_scan_enigma_murmur_modifier_dispose(GScanEnigmaMurmurModifier *modifier)
{
    G_OBJECT_CLASS(g_scan_enigma_murmur_modifier_parent_class)->dispose(G_OBJECT(modifier));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : modifier = instance d'objet GLib à traiter.                  *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_scan_enigma_murmur_modifier_finalize(GScanEnigmaMurmurModifier *modifier)
{
    G_OBJECT_CLASS(g_scan_enigma_murmur_modifier_parent_class)->finalize(G_OBJECT(modifier));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Construit un modificateur pour des empreintes enigma-murmur. *
*                                                                             *
*  Retour      : Mécanisme mis en place.                                      *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GScanTokenModifier *g_scan_enigma_murmur_modifier_new(void)
{
    GScanTokenModifier *result;                    /* Structure à retourner       */

    result = g_object_new(G_TYPE_SCAN_ENIGMA_MURMUR_MODIFIER, NULL);

    return result;

}



/* ---------------------------------------------------------------------------------- */
/*                       IMPLEMENTATION DES FONCTIONS DE CLASSE                       */
/* ---------------------------------------------------------------------------------- */


/******************************************************************************
*                                                                             *
*  Paramètres  : modifier = modificateur à consulter.                         *
*                                                                             *
*  Description : Fournit le nom d'appel d'un modificateur pour motif.         *
*                                                                             *
*  Retour      : Désignation humaine.                                         *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static char *g_scan_enigma_murmur_modifier_get_name(const GScanEnigmaMurmurModifier *modifier)
{
    char *result;                           /* Désignation à retourner     */

    result = strdup("enigma-murmur");

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : src = séquence d'octets à traiter.                           *
*                                                                             *
*  Description : Calcule l'empreinte enigma-murmur d'un motif de recherche.   *
*                                                                             *
*  Retour      : Valeur entière de l'empreinte déterminée.                    *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static uint32_t compute_enigma_murmur_hash32(const sized_binary_t *src)
{
    uint32_t result;                        /* Valeur à retourner          */
    size_t blk_count;                       /* Nombre de blocs présents    */
    size_t i;                               /* Boucle de parcours          */
    uint32_t k;                             /* Valeur pour un bloc         */
    const bin_t *tail;                      /* Fragement de bloc final     */

    result = 0x4a03bdfa;

    /* Traitement par blocs de 4 octets */

    blk_count = src->len / 4;

    for (i = 0; i < blk_count; i++)
    {
        k = ((uint32_t *)src->data)[i];

        k *= 0xcc9e2d51;
        k = (k << 15) | (k >> 17);
        k *= 0x1b873593;

        result ^= k;
        result = (result << 13) | (result >> 19);
        result = result * 5 + 0xe6546b64;

    }

    /* Traitement du reste */

    tail = src->static_bin_data + 4 * blk_count;

    k = 0;

    switch (src->len & 3)
    {
        case 3:
            k ^= (tail[2] << 16);

        case 2:
            k ^= (tail[1] << 8);

        case 1:
            k ^= tail[0];
            k *= 0xcc9e2d51;
            k = (k << 15) | (k >> (17));
            k *= 0x1b873593;
            result ^= k;
            break;

        case 0:
            break;

    }

    /* Conclusion */

    result ^= src->len;

    result ^= (result >> 16);
    result *= 0x85ebca6b;
    result ^= (result >> 13);
    result *= 0xc2b2ae35;
    result ^= (result >> 16);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : modifier = modificateur à solliciter.                        *
*                src      = séquences d'octets à traiter.                     *
*                scount   = quantité de ces séquences.                        *
*                dest     = nouvelle(s) séquence(s) d'octets obtenue(s) [OUT] *
*                dcount   = quantité de ces séquences.                        *
*                                                                             *
*  Description : Transforme une séquence d'octets pour motif de recherche.    *
*                                                                             *
*  Retour      : Bilan de l'opération : succès ou échec.                      *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static bool g_scan_enigma_murmur_modifier_transform(const GScanEnigmaMurmurModifier *modifier, const sized_binary_t *src, size_t scount, sized_binary_t **dest, size_t *dcount)
{
    bool result;                            /* Bilan d'opération à renvoyer*/
    sized_binary_t *binary;                 /* Raccourci vers le stockage  */
    size_t i;                               /* Boucle de parcours #1       */
    const sized_binary_t *_src;             /* Source courante             */
    uint32_t hash;                          /* Valeur d'empreinte calculée */

    result = true;

    *dcount = scount;
    *dest = calloc(*dcount, sizeof(sized_binary_t));

    binary = &(*dest)[0];

    for (i = 0; i < scount; i++, binary++)
    {
        _src = src + i;

        hash = compute_enigma_murmur_hash32(_src);

        binary->data = malloc(sizeof(hash) * sizeof(bin_t));
        binary->len = sizeof(hash);

        memcpy(binary->data, &hash, sizeof(hash));

    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : modifier = modificateur à consulter.                         *
*                index    = indice de la combinaison ciblée. [OUT]            *
*                                                                             *
*  Description : Retrouve l'origine d'une correspondance à partir d'un indice.*
*                                                                             *
*  Retour      : Version humainement lisible de la combinaison.               *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static char *g_scan_enigma_murmur_modifier_get_path(const GScanEnigmaMurmurModifier *modifier, size_t *index)
{
    char *result;                           /* Combinaison à retourner     */

    if (*index > 0)
    {
        result = NULL;
        (*index)--;
    }

    else
        result = strdup("enigma-murmur");

    return result;

}
